package org.mammon.scheme.brands.generic.coin;

import org.mammon.AssetType;
import org.mammon.math.FiniteField;
import org.mammon.math.Group;
import org.mammon.messaging.DualIdentityTransitionable;
import org.mammon.messaging.Identifiable;
import org.mammon.messaging.Transitionable;
import org.mammon.scheme.brands.BrandsSchemeSetup;
import org.mammon.scheme.brands.PaymentHashFunction;
import org.mammon.scheme.brands.SignatureHashFunction;
import org.mammon.scheme.brands.accountholder.AccountHolder;
import org.mammon.scheme.brands.accountholder.AccountHolderPrivate;
import org.mammon.scheme.brands.bank.Bank;
import org.mammon.scheme.brands.coin.CoinSignature;
import org.mammon.scheme.brands.coin.UnspentCoin;
import org.mammon.scheme.brands.generic.assettypes.EuroAssetType;
import org.mammon.scheme.brands.messages.IssueCoinsResponse;
import org.mammon.scheme.brands.messages.TransferToShopMessage;
import org.mammon.util.HashCodeUtil;
import org.mammon.util.messaging.AbstractTransitionable;

public abstract class AbstractUnspentCoin<G extends Group<G>, F extends FiniteField<F>, T, H extends SignatureHashFunction<G, F>, H0 extends PaymentHashFunction<G, F, T>>
		extends AbstractCoin<G, F, T, H, H0> implements UnspentCoin<G, F, T, H, H0>, Identifiable,
		Transitionable, DualIdentityTransitionable {

	private final AccountHolderPrivate<G, F, T, H, H0> bearer;

	private final String dualIdentity;

	private final FiniteField.Element<F> s;

	private final FiniteField.Element<F> x1;

	private final FiniteField.Element<F> x2;

	private final FiniteField.Element<F> r;

	protected AbstractUnspentCoin(BrandsSchemeSetup<G, F, T, H, H0> setup,
			AccountHolderPrivate<G, F, T, H, H0> bearer, Bank<G, F, T, H, H0> bank, String identity,
			String dualIdentity, FiniteField.Element<F> blindingFactor, FiniteField.Element<F> x1,
			FiniteField.Element<F> x2, Group.Element<G> blindedIdentity, Group.Element<G> commitment,
			FiniteField.Element<F> r, CoinSignature<G, F> coinSignature) {
		super(setup, bank, blindedIdentity, commitment, coinSignature, new EuroAssetType(), Integer
				.valueOf(1), identity);
		this.bearer = bearer;
		this.dualIdentity = dualIdentity;
		this.s = blindingFactor;
		this.x1 = x1;
		this.x2 = x2;
		this.r = r;
	}

	@Override
	public AccountHolder<G, F, T, H, H0> getBearer() {
		return bearer;
	}

	@Override
	public FiniteField.Element<F> getBlindingFactor() {
		return s;
	}

	@Override
	public FiniteField.Element<F> getPayerWitness1() {
		return x1;
	}

	@Override
	public FiniteField.Element<F> getPayerWitness2() {
		return x2;
	}

	public FiniteField.Element<F> getR() {
		return r;
	}

	@Override
	public boolean isSellable() {
		return true;
	}

	@Override
	public Transitionable getSecondaryTransitionable() {
		return new SecondaryTransitionable();
	}

	public AbstractTransferringCoinOne<G, F, T, H, H0> transition(TransferToShopMessage message) {
		return newAbstractTransferringCoinOne(getSetup(), getIssuer(), bearer, getBlindedIdentity(), getCommitment(),
				getCoinSignature(), getAssetType(), getFaceValue(), s, x1, x2, getIdentity(), message.getShop());
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == null || !(obj instanceof AbstractUnspentCoin<?, ?, ?, ?, ?>)) {
			return false;
		}
		AbstractUnspentCoin<?, ?, ?, ?, ?> other = (AbstractUnspentCoin<?, ?, ?, ?, ?>) obj;
		return getSetup().equals(other.getSetup()) && getIssuer().equals(other.getIssuer())
				&& bearer.equals(other.bearer) && s.equals(other.s) && x1.equals(other.x1) && x2.equals(other.x2);
	}

	@Override
	public int hashCode() {
		int hashCode = HashCodeUtil.SEED;
		hashCode = HashCodeUtil.hash(hashCode, getSetup());
		hashCode = HashCodeUtil.hash(hashCode, getIssuer());
		hashCode = HashCodeUtil.hash(hashCode, bearer);
		hashCode = HashCodeUtil.hash(hashCode, s);
		hashCode = HashCodeUtil.hash(hashCode, x1);
		hashCode = HashCodeUtil.hash(hashCode, x2);
		return hashCode;
	}

	protected final class SecondaryTransitionable extends AbstractTransitionable {
		@Override
		public String getIdentity() {
			return dualIdentity;
		}

		public Object transition(IssueCoinsResponse<F> response) {
			if (r.equals(response.getResponse())) {
				return AbstractUnspentCoin.this;
			} else {
				return null;
			}
		}
	}

	protected abstract AbstractTransferringCoinOne<G, F, T, H, H0> newAbstractTransferringCoinOne(
			BrandsSchemeSetup<G, F, T, H, H0> setup, Bank<G, F, T, H, H0> bank,
			AccountHolderPrivate<G, F, T, H, H0> bearer, Group.Element<G> blindedIdentity,
			Group.Element<G> commitment, CoinSignature<G, F> coinSignature, AssetType assetType, Number faceValue,
			FiniteField.Element<F> s, FiniteField.Element<F> x1, FiniteField.Element<F> x2, String identity, String shop);

}
